@@ -20,29 +20,20 @@ bool TestStable(const arma::mat& x);
 double DirectStd(const arma::mat& phi, const arma::mat& sigma,
                  const double& delta_t, const arma::uword& from,
                  const arma::uword& to, const arma::vec& med) {
-  arma::mat I = arma::eye(phi.n_rows, phi.n_cols);
-  arma::mat J = arma::eye(phi.n_rows * phi.n_cols, phi.n_rows * phi.n_cols);
   arma::mat d = arma::eye(phi.n_rows, phi.n_rows);
   for (arma::uword i = 0; i < med.n_elem; ++i) {
     d(med[i] - 1, med[i] - 1) = 0;
   }
-  arma::mat total = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  // arma::vec psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-  // delta_t) - J) * sigma_vec;
-  arma::vec psi_vec = arma::solve(
-      phi_hashtag, (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec);
-  // arma::mat total_cov = arma::reshape(arma::inv(J - arma::kron(total, total))
-  // * psi_vec, phi.n_rows, phi.n_cols);
-  arma::mat total_cov =
-      arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                    phi.n_rows, phi.n_cols);
-  arma::mat sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-  arma::mat sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
   arma::mat direct = arma::expmat(delta_t * d * phi * d);
-  // arma::mat direct_std = d * (sd_row * direct * sd_col_inv) * d;
-  arma::mat direct_std = sd_row * direct * sd_col_inv;
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
+  arma::mat direct_std = direct;
+  for (size_t i = 0; i < direct.n_rows; i++) {
+    for (size_t j = 0; j < direct.n_cols; j++) {
+      direct_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   return direct_std(to - 1, from - 1);
 }
 // -----------------------------------------------------------------------------
@@ -64,46 +55,6 @@ double Direct(const arma::mat& phi, const double& delta_t,
   return direct(to - 1, from - 1);
 }
 // -----------------------------------------------------------------------------
-// edit .setup/cpp/cTMed-exp-cov.cpp
-// Ivan Jacob Agaloos Pesigan
-// -----------------------------------------------------------------------------
-
-#include <RcppArmadillo.h>
-// [[Rcpp::depends(RcppArmadillo)]]
-// [[Rcpp::export(.ExpCov)]]
-arma::mat ExpCov(const arma::mat& phi, const arma::mat& sigma,
-                 const double& delta_t) {
-  arma::mat I = arma::eye(phi.n_rows, phi.n_cols);
-  arma::mat J = arma::eye(phi.n_rows * phi.n_cols, phi.n_rows * phi.n_cols);
-  arma::mat beta = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  arma::vec psi_vec = arma::inv(phi_hashtag) *
-                      (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec;
-  return arma::reshape(arma::inv(J - arma::kron(beta, beta)) * psi_vec,
-                       phi.n_rows, phi.n_cols);
-}
-// -----------------------------------------------------------------------------
-// edit .setup/cpp/cTMed-exp-mean.cpp
-// Ivan Jacob Agaloos Pesigan
-// -----------------------------------------------------------------------------
-
-#include <RcppArmadillo.h>
-// [[Rcpp::depends(RcppArmadillo)]]
-// [[Rcpp::export(.ExpMean)]]
-Rcpp::NumericVector ExpMean(const arma::mat& phi, const arma::vec& iota,
-                            const double& delta_t) {
-  if (arma::all(iota == 0)) {
-    return Rcpp::NumericVector(phi.n_rows, 0.0);
-  }
-  arma::mat I = arma::eye(phi.n_rows, phi.n_cols);
-  arma::mat beta = arma::expmat(delta_t * phi);
-  arma::vec alpha = arma::solve(phi, (beta - I) * iota);
-  arma::vec mu = arma::solve((I - beta), alpha);
-  Rcpp::NumericVector output(mu.begin(), mu.end());
-  return output;
-}
-// -----------------------------------------------------------------------------
 // edit .setup/cpp/cTMed-indirect-central-s.cpp
 // Ivan Jacob Agaloos Pesigan
 // -----------------------------------------------------------------------------
@@ -203,31 +154,28 @@ Rcpp::NumericVector IndirectCentral(const arma::mat& phi,
 double IndirectStd(const arma::mat& phi, const arma::mat& sigma,
                    const double& delta_t, const arma::uword& from,
                    const arma::uword& to, const arma::vec& med) {
-  arma::mat I = arma::eye(phi.n_rows, phi.n_cols);
-  arma::mat J = arma::eye(phi.n_rows * phi.n_cols, phi.n_rows * phi.n_cols);
   arma::mat d = arma::eye(phi.n_rows, phi.n_rows);
   for (arma::uword i = 0; i < med.n_elem; ++i) {
     d(med[i] - 1, med[i] - 1) = 0;
   }
   arma::mat total = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  // arma::vec psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-  // delta_t) - J) * sigma_vec;
-  arma::vec psi_vec = arma::solve(
-      phi_hashtag, (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec);
-  // arma::mat total_cov = arma::reshape(arma::inv(J - arma::kron(total, total))
-  // * psi_vec, phi.n_rows, phi.n_cols);
-  arma::mat total_cov =
-      arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                    phi.n_rows, phi.n_cols);
-  arma::mat sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-  arma::mat sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
-  arma::mat total_std = sd_row * total * sd_col_inv;
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
+  arma::mat total_std = total;
+  for (size_t i = 0; i < total.n_rows; i++) {
+    for (size_t j = 0; j < total.n_cols; j++) {
+      total_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   double total_dbl = total_std(to - 1, from - 1);
   arma::mat direct = arma::expmat(delta_t * d * phi * d);
-  // arma::mat direct_std = d * (sd_row * direct * sd_col_inv) * d;
-  arma::mat direct_std = sd_row * direct * sd_col_inv;
+  arma::mat direct_std = direct;
+  for (size_t i = 0; i < direct.n_rows; i++) {
+    for (size_t j = 0; j < direct.n_cols; j++) {
+      direct_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   double direct_dbl = direct_std(to - 1, from - 1);
   return total_dbl - direct_dbl;
 }
@@ -492,32 +440,29 @@ arma::mat MedStds(const arma::mat& phi, const arma::mat& sigma,
   for (arma::uword i = 0; i < med.n_elem; ++i) {
     d(med[i] - 1, med[i] - 1) = 0;
   }
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  arma::vec psi_vec(phi.n_rows * phi.n_cols, arma::fill::none);
-  arma::mat total_cov(phi.n_rows, phi.n_cols, arma::fill::none);
   arma::mat sd_row(phi.n_rows, phi.n_cols, arma::fill::none);
   arma::mat sd_col_inv(phi.n_rows, phi.n_cols, arma::fill::none);
   arma::mat total_std(phi.n_rows, phi.n_cols, arma::fill::none);
   arma::mat direct_std(phi.n_rows, phi.n_cols, arma::fill::none);
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
   for (arma::uword t = 0; t < delta_t.n_elem; t++) {
     total = arma::expmat(delta_t[t] * phi);
-    // psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-    // delta_t[t]) - J) * sigma_vec;
-    psi_vec = arma::solve(
-        phi_hashtag, (arma::expmat(phi_hashtag * delta_t[t]) - J) * sigma_vec);
-    // total_cov = arma::reshape(arma::inv(J - arma::kron(total, total)) *
-    // psi_vec, phi.n_rows, phi.n_cols);
-    total_cov =
-        arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                      phi.n_rows, phi.n_cols);
-    sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-    sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
-    total_std = sd_row * total * sd_col_inv;
+    arma::mat total_std = total;
+    for (size_t i = 0; i < total.n_rows; i++) {
+      for (size_t j = 0; j < total.n_cols; j++) {
+        total_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+      }
+    }
     total_dbl = total_std(to - 1, from - 1);
     direct = arma::expmat(delta_t[t] * d * phi * d);
-    // direct_std = d * (sd_row * direct * sd_col_inv) * d;
-    direct_std = sd_row * direct * sd_col_inv;
+    arma::mat direct_std = direct;
+    for (size_t i = 0; i < direct.n_rows; i++) {
+      for (size_t j = 0; j < direct.n_cols; j++) {
+        direct_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+      }
+    }
     direct_dbl = direct_std(to - 1, from - 1);
     indirect_dbl = total_dbl - direct_dbl;
     output(t, 0) = total_dbl;
@@ -557,27 +502,24 @@ Rcpp::NumericVector MedStdVec(const arma::vec& v, const double& delta_t,
   for (arma::uword i = 0; i < med.n_elem; ++i) {
     d(med[i] - 1, med[i] - 1) = 0;
   }
-  arma::mat I = arma::eye(p, p);
-  arma::mat J = arma::eye(p * p, p * p);
   arma::mat total = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  // arma::vec psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-  // delta_t) - J) * sigma_vec;
-  arma::vec psi_vec = arma::solve(
-      phi_hashtag, (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec);
-  // arma::mat total_cov = arma::reshape(arma::inv(J - arma::kron(total, total))
-  // * psi_vec, phi.n_rows, phi.n_cols);
-  arma::mat total_cov =
-      arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                    phi.n_rows, phi.n_cols);
-  arma::mat sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-  arma::mat sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
-  arma::mat total_std = sd_row * total * sd_col_inv;
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
+  arma::mat total_std = total;
+  for (size_t i = 0; i < total.n_rows; i++) {
+    for (size_t j = 0; j < total.n_cols; j++) {
+      total_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   double total_dbl = total_std(to - 1, from - 1);
   arma::mat direct = arma::expmat(delta_t * d * phi * d);
-  // arma::mat direct_std = d * (sd_row * direct * sd_col_inv) * d;
-  arma::mat direct_std = sd_row * direct * sd_col_inv;
+  arma::mat direct_std = direct;
+  for (size_t i = 0; i < direct.n_rows; i++) {
+    for (size_t j = 0; j < direct.n_cols; j++) {
+      direct_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   double direct_dbl = direct_std(to - 1, from - 1);
   double indirect_dbl = total_dbl - direct_dbl;
   Rcpp::NumericVector output(3);
@@ -597,31 +539,28 @@ Rcpp::NumericVector MedStdVec(const arma::vec& v, const double& delta_t,
 Rcpp::NumericVector MedStd(const arma::mat& phi, const arma::mat& sigma,
                            const double& delta_t, const arma::uword& from,
                            const arma::uword& to, const arma::vec& med) {
-  arma::mat I = arma::eye(phi.n_rows, phi.n_cols);
-  arma::mat J = arma::eye(phi.n_rows * phi.n_cols, phi.n_rows * phi.n_cols);
   arma::mat d = arma::eye(phi.n_rows, phi.n_rows);
   for (arma::uword i = 0; i < med.n_elem; ++i) {
     d(med[i] - 1, med[i] - 1) = 0;
   }
   arma::mat total = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  // arma::vec psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-  // delta_t) - J) * sigma_vec;
-  arma::vec psi_vec = arma::solve(
-      phi_hashtag, (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec);
-  // arma::mat total_cov = arma::reshape(arma::inv(J - arma::kron(total, total))
-  // * psi_vec, phi.n_rows, phi.n_cols);
-  arma::mat total_cov =
-      arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                    phi.n_rows, phi.n_cols);
-  arma::mat sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-  arma::mat sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
-  arma::mat total_std = sd_row * total * sd_col_inv;
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
+  arma::mat total_std = total;
+  for (size_t i = 0; i < total.n_rows; i++) {
+    for (size_t j = 0; j < total.n_cols; j++) {
+      total_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   double total_dbl = total_std(to - 1, from - 1);
   arma::mat direct = arma::expmat(delta_t * d * phi * d);
-  // arma::mat direct_std = d * (sd_row * direct * sd_col_inv) * d;
-  arma::mat direct_std = sd_row * direct * sd_col_inv;
+  arma::mat direct_std = direct;
+  for (size_t i = 0; i < direct.n_rows; i++) {
+    for (size_t j = 0; j < direct.n_cols; j++) {
+      direct_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   double direct_dbl = direct_std(to - 1, from - 1);
   double indirect_dbl = total_dbl - direct_dbl;
   Rcpp::NumericVector output(4);
@@ -783,23 +722,16 @@ Rcpp::NumericVector TotalDeltaT(const arma::mat& phi, const double& delta_t) {
 // [[Rcpp::export(.TotalStdDeltaT)]]
 Rcpp::NumericVector TotalStdDeltaT(const arma::mat& phi, const arma::mat& sigma,
                                    const double& delta_t) {
-  arma::mat I = arma::eye(phi.n_rows, phi.n_cols);
-  arma::mat J = arma::eye(phi.n_rows * phi.n_cols, phi.n_rows * phi.n_cols);
   arma::mat total = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  // arma::vec psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-  // delta_t) - J) * sigma_vec;
-  arma::vec psi_vec = arma::solve(
-      phi_hashtag, (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec);
-  // arma::mat total_cov = arma::reshape(arma::inv(J - arma::kron(total, total))
-  // * psi_vec, phi.n_rows, phi.n_cols);
-  arma::mat total_cov =
-      arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                    phi.n_rows, phi.n_cols);
-  arma::mat sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-  arma::mat sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
-  arma::mat total_std = sd_row * total * sd_col_inv;
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
+  arma::mat total_std = total;
+  for (size_t i = 0; i < total.n_rows; i++) {
+    for (size_t j = 0; j < total.n_cols; j++) {
+      total_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   Rcpp::NumericVector total_std_vec(total_std.memptr(),
                                     total_std.memptr() + total_std.n_elem);
   total_std_vec.push_back(delta_t);
@@ -829,23 +761,16 @@ arma::vec TotalStdVec(const arma::vec& v, const double& delta_t) {
       index++;
     }
   }
-  arma::mat I = arma::eye(p, p);
-  arma::mat J = arma::eye(p * p, p * p);
   arma::mat total = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  // arma::vec psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-  // delta_t) - J) * sigma_vec;
-  arma::vec psi_vec = arma::solve(
-      phi_hashtag, (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec);
-  // arma::mat total_cov = arma::reshape(arma::inv(J - arma::kron(total, total))
-  // * psi_vec, phi.n_rows, phi.n_cols);
-  arma::mat total_cov =
-      arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                    phi.n_rows, phi.n_cols);
-  arma::mat sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-  arma::mat sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
-  arma::mat total_std = sd_row * total * sd_col_inv;
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
+  arma::mat total_std = total;
+  for (size_t i = 0; i < total.n_rows; i++) {
+    for (size_t j = 0; j < total.n_cols; j++) {
+      total_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
   return arma::vectorise(total_std);
 }
 // -----------------------------------------------------------------------------
@@ -858,23 +783,17 @@ arma::vec TotalStdVec(const arma::vec& v, const double& delta_t) {
 // [[Rcpp::export(.TotalStd)]]
 arma::mat TotalStd(const arma::mat& phi, const arma::mat& sigma,
                    const double& delta_t) {
-  arma::mat I = arma::eye(phi.n_rows, phi.n_cols);
-  arma::mat J = arma::eye(phi.n_rows * phi.n_cols, phi.n_rows * phi.n_cols);
   arma::mat total = arma::expmat(delta_t * phi);
-  arma::mat phi_hashtag = arma::kron(phi, I) + arma::kron(I, phi);
-  arma::vec sigma_vec = arma::vectorise(sigma);
-  // arma::vec psi_vec = arma::inv(phi_hashtag) * (arma::expmat(phi_hashtag *
-  // delta_t) - J) * sigma_vec;
-  arma::vec psi_vec = arma::solve(
-      phi_hashtag, (arma::expmat(phi_hashtag * delta_t) - J) * sigma_vec);
-  // arma::mat total_cov = arma::reshape(arma::inv(J - arma::kron(total, total))
-  // * psi_vec, phi.n_rows, phi.n_cols);
-  arma::mat total_cov =
-      arma::reshape(arma::solve(J - arma::kron(total, total), psi_vec),
-                    phi.n_rows, phi.n_cols);
-  arma::mat sd_row = arma::diagmat(arma::sqrt(total_cov.diag()));
-  arma::mat sd_col_inv = arma::diagmat(1.0 / arma::sqrt(total_cov.diag()));
-  return sd_row * total * sd_col_inv;
+  arma::mat cov_eta;
+  arma::syl(cov_eta, phi, phi.t(), sigma);
+  arma::vec sqrt_diag = arma::sqrt(cov_eta.diag());
+  arma::mat total_std = total;
+  for (size_t i = 0; i < total.n_rows; i++) {
+    for (size_t j = 0; j < total.n_cols; j++) {
+      total_std(i, j) *= sqrt_diag(j) / sqrt_diag(i);
+    }
+  }
+  return total_std;
 }
 // -----------------------------------------------------------------------------
 // edit .setup/cpp/cTMed-total-vec.cpp
